--*******************************************************************************************************************************
--
-- Description: This module sends data to a variety of D/A converter(DAC) chips.  The bus type supports multiple TX channels
--				and stuffing of multiple samples per clock cyle to accomodate DDR outputs.  When samples are stuffed, they are
--				chronologically ordered from LSB to MSB.
--					
-- Rev History: 09-04-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.sigfpga_package.all;

entity dac_interface is 
	port (
    -- Reset and clock.                                                                                      
	rst         	: in std_logic;		--	Active high asynchronous reset.       
	tx_clk			: in std_logic;		--	Clock used to register DAC sample data.
	
	--	The following signals connect to DAC devices through FPGA pins.
	tx_data_in		: in TX_BUS;	--  Data bus carrying DAC samples.
	tx_data_out		: out TX_BUS;	--  Data bus carrying DAC samples.
	tx_clk_out		: out TX_1		-- Internal single ended version of DAC forwarded clock.

	);	
	end dac_interface;	

architecture behavioral of dac_interface is

begin                    
 	
--************************************************** EXTERNAL PIN ASSIGNMENTS **************************************************

	-- Instantiate a SDR data output bus.
	SDR : if TX_DATA_DDR = false generate
		SDR_CHANNEL : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			SDR_DATA : for j in 0 to TX_ITEM_SIZE-1 generate
				-- Use double data rate (DDR) output registers in the IOB to transmit data on the rising and falling edge of the sample clock.
				-- Note: The DDR register is used on SDR outputs to ensure that the signal is registered inside the IOB.
				DATA_SDR : ODDR
					GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE") 
					PORT MAP(Q => tx_data_out(i)(j), C => tx_clk, CE => '1', D1 => tx_data_in(i)(j), D2 => tx_data_in(i)(j), R => rst, S => '0');
			end generate;
		end generate;
	end generate;
	
	-- Instantiate a DDR data output bus.
	DDR : if TX_DATA_DDR = true generate
		DDR_CHANNEL : for i in RX_CHANNEL_COUNT+1 to integer(TX_CHANNEL_COUNT/2)+RX_CHANNEL_COUNT generate
			DDR_DATA : for j in 0 to TX_ITEM_SIZE-1 generate
				-- Use double data rate (DDR) output registers in the IOB to transmit data on the rising and falling edge of the sample clock.
				-- Note: The DDR register is used on SDR outputs to ensure that the signal is registered inside the IOB.
				DATA_DDR : ODDR
					GENERIC MAP(DDR_CLK_EDGE => "SAME_EDGE") 
					PORT MAP(Q => tx_data_out(i)(j), C => tx_clk, CE => '1', D1 => tx_data_in(2*i-1)(j), D2 => tx_data_in(2*i)(j), R => rst, S => '0');
			end generate;
	
			-- Instantiate a DDR clock forward if requested.
			CLK_FWD : if TX_CLK_FWD = true generate
				-- Use a double data rate (DDR) register to generate the forwarded DAC clock.
				DCLK_ODDR : ODDR
					generic map (DDR_CLK_EDGE => "SAME_EDGE") 
					port map (D1 => '1', D2 => '0', C => tx_clk, CE => '1', Q => tx_clk_out(i), R => rst, S => '0');
			end generate;
		end generate;
	end generate;
	
end behavioral;
