--*******************************************************************************************************************************
--
-- Description: This package declares all of the unique data types used in the SigFPGA reference design.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;

package sigfpga_package is

	-- Define the multi-bus data types.
	type DUAL_1 is array (1 downto 0) of std_logic;
	type DUAL_1SLV is array (1 downto 0) of std_logic_vector(0 downto 0);
	type DUAL_2 is array (1 downto 0) of std_logic_vector(1 downto 0);
	type DUAL_8 is array (1 downto 0) of std_logic_vector(7 downto 0);
	type DUAL_18 is array (1 downto 0) of std_logic_vector(17 downto 0);
	type DUAL_32 is array (1 downto 0) of std_logic_vector(31 downto 0);
	type DUAL_64 is array (1 downto 0) of std_logic_vector(63 downto 0);
	type DUAL_72 is array (1 downto 0) of std_logic_vector(71 downto 0);
	type DUAL_CHU is array (1 downto 0) of unsigned(TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1);
	type DUAL_QDR is array (1 downto 0) of std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);
	type AXI_11U is array ((AXI_WIDTH/64)-1 downto 0) of unsigned(10 downto 0);

	-- Define the DMA channel data types.
	type DMA_TYPE_4 is array (7 downto 0) of std_logic_vector(3 downto 0);
	type DMA_TYPE_6 is array (7 downto 0) of std_logic_vector(5 downto 0);
	type DMA_TYPE_13 is array (7 downto 0) of std_logic_vector(12 downto 0);
	type DMA_TYPE_24 is array (7 downto 0) of std_logic_vector(23 downto 0);
	type DMA_TYPE_128 is array (7 downto 0) of std_logic_vector(127 downto 0);
	type DMA_TYPE_AXI is array (7 downto 0) of std_logic_vector(AXI_WIDTH-1 downto 0);
	
	-- Define the four channel data types.
	type CH_1TO4_3 is array (4 downto 1) of std_logic_vector(2 downto 0);
	type CH_1TO4_4 is array (4 downto 1) of std_logic_vector(3 downto 0);
	type CH_1TO4_6 is array (4 downto 1) of std_logic_vector(5 downto 0);
	type CH_1TO4_10 is array (4 downto 1) of std_logic_vector(9 downto 0);
	type CH_1TO4_13 is array (4 downto 1) of std_logic_vector(12 downto 0);
	type CH_1TO4_16 is array (4 downto 1) of std_logic_vector(15 downto 0);
	type CH_1TO4_24 is array (4 downto 1) of std_logic_vector(23 downto 0);
	type CH_1TO4_64 is array (4 downto 1) of std_logic_vector(63 downto 0);
	type CH_1TO4_128 is array (4 downto 1) of std_logic_vector(127 downto 0);
	type CH_1TO4_AXI is array (4 downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);
	
	-- Define the receiver (RX) channel data types.
	type RX_BUS_IN is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);
	type RX_BUS is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic_vector((RX_ITEM_STUFF*RX_ITEM_SIZE)-1 downto 0);
	type OVR_BIT is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic_vector(RX_ITEM_STUFF-1 downto 0);
	type RX_1 is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic;

	-- Define the transmitter (TX) channel data types.
	type TX_BUS is array (zero_to_one(TX_CHANNEL_COUNT)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic_vector((TX_ITEM_STUFF*TX_ITEM_SIZE)-1 downto 0);
	type TX_1 is array (zero_to_one(TX_CHANNEL_COUNT)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic;
	
	-- Define the combined transmitter and receiver channel data types.	
	type CH_TYPE_1 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic;
	type CH_TYPE_2 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(1 downto 0);
	type CH_TYPE_3 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(2 downto 0);
	type CH_TYPE_4 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(3 downto 0);
	type CH_TYPE_6 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(5 downto 0);
	type CH_TYPE_10 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(9 downto 0);
	type CH_TYPE_13 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(12 downto 0);
	type CH_TYPE_16 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(15 downto 0);
	type CH_TYPE_24 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(23 downto 0);
	type CH_TYPE_32 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(31 downto 0);
	type CH_TYPE_36 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(35 downto 0);
	type CH_TYPE_64 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(63 downto 0);
	type CH_TYPE_128 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(127 downto 0);
	type CH_TYPE_AXI is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);
	
    -- PLDA EZDMA licensed core.
	component plda_ezdma2 is
		port (
		clk                     : in    std_logic;
		rstn                    : in    std_logic;
		test_mode               : in    std_logic_vector (15 downto 0);
		user_lnk_up             : in    std_logic;
		s_axis_tx_tdata         : out   std_logic_vector (127 downto 0);
		s_axis_tx_tkeep         : out   std_logic_vector (15 downto 0);
		s_axis_tx_tlast         : out   std_logic;
		s_axis_tx_tvalid        : out   std_logic;
		s_axis_tx_tready        : in    std_logic;
		s_axis_tx_tuser         : out   std_logic_vector (3 downto 0);
		tx_err_drop             : in    std_logic;
		tbuf_av                 : in    std_logic_vector (5 downto 0);
		m_axis_rx_tdata         : in    std_logic_vector (127 downto 0);
		m_axis_rx_tkeep         : in    std_logic_vector (15 downto 0);
		m_axis_rx_tlast         : in    std_logic;
		m_axis_rx_tvalid        : in    std_logic;
		m_axis_rx_tuser         : in    std_logic_vector (21 downto 0);
		m_axis_rx_tready        : out   std_logic;
		rx_rnp_ok               : out   std_logic;
		cfg_err_cor             : out   std_logic;
		cfg_err_cpl_abort       : out   std_logic;
		cfg_err_cpl_timeout     : out   std_logic;
		cfg_err_cpl_unexpect    : out   std_logic;
		cfg_err_ecrc            : out   std_logic;
		cfg_err_posted          : out   std_logic;
		cfg_err_tlp_cpl_header  : out   std_logic_vector (47 downto 0);
		cfg_err_ur              : out   std_logic;
		cfg_err_cpl_rdy         : in    std_logic;
		cfg_err_locked          : out   std_logic;
		cfg_interrupt           : out   std_logic;
		cfg_interrupt_rdy       : in    std_logic;
		cfg_pcie_link_state     : in    std_logic_vector (2 downto 0);
		cfg_interrupt_assert    : out   std_logic;
		cfg_interrupt_di        : out   std_logic_vector (7 downto 0);
		cfg_interrupt_do        : in    std_logic_vector (7 downto 0);
		cfg_interrupt_mmenable  : in    std_logic_vector (2 downto 0);
		cfg_interrupt_msienable : in    std_logic;
		cfg_trn_pending         : out   std_logic;
		cfg_bus_number          : in    std_logic_vector (7 downto 0);
		cfg_device_number       : in    std_logic_vector (4 downto 0);
		cfg_function_number     : in    std_logic_vector (2 downto 0);
		cfg_status              : in    std_logic_vector (15 downto 0);
		cfg_command             : in    std_logic_vector (15 downto 0);
		cfg_dstatus             : in    std_logic_vector (15 downto 0);
		cfg_dcommand            : in    std_logic_vector (15 downto 0);
		cfg_lstatus             : in    std_logic_vector (15 downto 0);
		cfg_lcommand            : in    std_logic_vector (15 downto 0);
		int_request             : in    std_logic;
		int_ack                 : out   std_logic;
		int_msgnum              : in    std_logic_vector (4 downto 0);
		cfg_prmcsr              : out   std_logic_vector (31 downto 0);
		cfg_devcsr              : out   std_logic_vector (31 downto 0);
		cfg_linkcsr             : out   std_logic_vector (31 downto 0);
		cfg_msicsr              : out   std_logic_vector (15 downto 0);
		cfg_ltssm               : out   std_logic_vector (4 downto 0);
		slv_dataout             : out   std_logic_vector (127 downto 0);
		slv_bytevalid           : out   std_logic_vector (15 downto 0);
		slv_bytecount           : out   std_logic_vector (12 downto 0);
		slv_dwcount             : out   std_logic_vector (10 downto 0);
		slv_addr                : out   std_logic_vector (63 downto 0);
		slv_bar                 : out   std_logic_vector (6 downto 0);
		slv_readreq             : out   std_logic;
		slv_cpladdr             : out   std_logic_vector (31 downto 0);
		slv_cplparam            : out   std_logic_vector (7 downto 0);
		slv_writereq            : out   std_logic;
		slv_write               : out   std_logic;
		slv_lastwrite           : out   std_logic;
		slv_io                  : out   std_logic;
		slv_accept              : in    std_logic;
		slv_abort               : in    std_logic;
		slv_ur                  : in    std_logic;
		dma_rd                  : out   std_logic;
		dma_rdaddr              : out   std_logic_vector (15 downto 0);
		dma_rdchannel           : out   std_logic_vector (15 downto 0);
		dma_rddata              : in    std_logic_vector (127 downto 0);
		dma_wr                  : out   std_logic;
		dma_wraddr              : out   std_logic_vector (15 downto 0);
		dma_wrchannel           : out   std_logic_vector (15 downto 0);
		dma_wrdata              : out   std_logic_vector (127 downto 0);
		dma_wrbytevalid         : out   std_logic_vector (15 downto 0);
		dma0_regin              : in    std_logic_vector (127 downto 0);
		dma0_regout             : out   std_logic_vector (127 downto 0);
		dma0_param              : in    std_logic_vector (23 downto 0);
		dma0_control            : in    std_logic_vector (5 downto 0);
		dma0_status             : out   std_logic_vector (3 downto 0);
		dma0_fifocnt            : in    std_logic_vector (12 downto 0);
		dma1_regin              : in    std_logic_vector (127 downto 0);
		dma1_regout             : out   std_logic_vector (127 downto 0);
		dma1_param              : in    std_logic_vector (23 downto 0);
		dma1_control            : in    std_logic_vector (5 downto 0);
		dma1_status             : out   std_logic_vector (3 downto 0);
		dma1_fifocnt            : in    std_logic_vector (12 downto 0);
		dma2_regin              : in    std_logic_vector (127 downto 0);
		dma2_regout             : out   std_logic_vector (127 downto 0);
		dma2_param              : in    std_logic_vector (23 downto 0);
		dma2_control            : in    std_logic_vector (5 downto 0);
		dma2_status             : out   std_logic_vector (3 downto 0);
		dma2_fifocnt            : in    std_logic_vector (12 downto 0);
		dma3_regin              : in    std_logic_vector (127 downto 0);
		dma3_regout             : out   std_logic_vector (127 downto 0);
		dma3_param              : in    std_logic_vector (23 downto 0);
		dma3_control            : in    std_logic_vector (5 downto 0);
		dma3_status             : out   std_logic_vector (3 downto 0);
		dma3_fifocnt            : in    std_logic_vector (12 downto 0);
		dma4_regin              : in    std_logic_vector (127 downto 0);
		dma4_regout             : out   std_logic_vector (127 downto 0);
		dma4_param              : in    std_logic_vector (23 downto 0);
		dma4_control            : in    std_logic_vector (5 downto 0);
		dma4_status             : out   std_logic_vector (3 downto 0);
		dma4_fifocnt            : in    std_logic_vector (12 downto 0);
		dma5_regin              : in    std_logic_vector (127 downto 0);
		dma5_regout             : out   std_logic_vector (127 downto 0);
		dma5_param              : in    std_logic_vector (23 downto 0);
		dma5_control            : in    std_logic_vector (5 downto 0);
		dma5_status             : out   std_logic_vector (3 downto 0);
		dma5_fifocnt            : in    std_logic_vector (12 downto 0);
		dma6_regin              : in    std_logic_vector (127 downto 0);
		dma6_regout             : out   std_logic_vector (127 downto 0);
		dma6_param              : in    std_logic_vector (23 downto 0);
		dma6_control            : in    std_logic_vector (5 downto 0);
		dma6_status             : out   std_logic_vector (3 downto 0);
		dma6_fifocnt            : in    std_logic_vector (12 downto 0);
		dma7_regin              : in    std_logic_vector (127 downto 0);
		dma7_regout             : out   std_logic_vector (127 downto 0);
		dma7_param              : in    std_logic_vector (23 downto 0);
		dma7_control            : in    std_logic_vector (5 downto 0);
		dma7_status             : out   std_logic_vector (3 downto 0);
		dma7_fifocnt            : in    std_logic_vector (12 downto 0)
		);
	end component;
	
end sigfpga_package;
